<?php

declare(strict_types=1);

namespace Drupal\batch\Batch;

/**
 * Defines a batch that processes over an enumerated list of items.
 */
abstract class EnumeratedBatchBase extends BatchBase {

  /**
   * Items which the batch will iterate over.
   *
   * @var array
   */
  protected array $items = [];

  /**
   * Process one item from the batch.
   *
   * @param mixed $item
   *   The item to be processed.
   * @param array $context
   *   The batch context.
   */
  abstract public function processItem(mixed $item, array &$context);

  /**
   * {@inheritdoc}
   */
  public function process(&$context) {
    if (empty($context['sandbox'])) {
      $context['sandbox']['items'] = $this->items;
      $context['sandbox']['total'] = count($context['sandbox']['items']);
    }

    $processed = 0;
    while ($processed++ < $this->itemsPerProcess) {
      if ($item = array_shift($context['sandbox']['items'])) {
        $this->processItem($item, $context);
      }

      // If none left, we're done.
      if (empty($context['sandbox']['items'])) {
        return;
      }
    }

    if ($this->reclaimMemory()) {
      $context['message'] .= ' - Reclaiming memory';
    }

    $count = count($context['sandbox']['items']);
    $context['finished'] = 1 - ($count / $context['sandbox']['total']);
    $context['finished'] = min($context['finished'], .9999);
  }

}
